using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace Pitt_Lab_Printer_Setup
{
    public partial class AppGUI : Form
    {
        private delegate void MessageUpdater(string error);
        private delegate void StatusMsgUpdater(string caption, string msg);

        static AppGUI()
        {
            AppDomain.CurrentDomain.UnhandledException += delegate(object sender, UnhandledExceptionEventArgs e) {
                string errorMessage = "Error occurred: " + (e.ExceptionObject as Exception).ToString();
                try
                {
                    InstallLogger.Instance.Write(errorMessage);
                    InstallLogger.Instance.Close();
                }
                catch (Exception) { }
                MessageBox.Show(errorMessage);
                Application.Exit();
            };
        }

        public AppGUI()
        {
            InitializeComponent();
            base.Load += new EventHandler(OnLoad);
        }

        protected void OnLoad(Object sender, EventArgs e)
        {
            if (!Utilities.IsAdmin)
            {
                this.showErrorMessageBox("This program must be run as Administrator.");
                Environment.Exit(-1);
            }
            if (!Utilities.IsCompatibleOS)
            {
                this.showErrorMessageBox("This program only works on Windows 2000 and later.");
                Environment.Exit(-1);
            }
            this.modifyButtonStates();
        }

        private void modifyButtonStates()
        {
            MethodInvoker modBtns = delegate()
            {
                this.button1.Enabled = !Utilities.IsPrinterInstalled();
                this.button2.Enabled = Utilities.IsPrinterInstalled();
                this.prgBar.Value = 0;
            };
            if (InvokeRequired)
            {
                this.Invoke(modBtns);
            }
            else
            {
                modBtns.Invoke();
            }
        }

        private void showErrorMessageBox(string error)
        {
            MethodInvoker mi = delegate()
            {
                MessageBox.Show(this, error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.modifyButtonStates();
            };

            if (InvokeRequired)
            {
                this.Invoke(mi);
            }
            else
            {
                mi.Invoke();
            }
        }

        private void showInfoMessageBox(string caption, string msg)
        {
            MethodInvoker mi = delegate()
            {
                MessageBox.Show(this, msg, caption, MessageBoxButtons.OK, MessageBoxIcon.Information);
            };

            if (this.InvokeRequired)
            {
                this.Invoke(mi);
            }
            else
            {
               mi.Invoke();
            }

        }

        private void updateProgress()
        {
            MethodInvoker mi = delegate()
            {
                this.prgBar.PerformStep();
            };
            if (this.InvokeRequired)
            {
                this.Invoke(mi);
            }
            else
            {
                mi.Invoke();
            }

        }

        private void Install(object o)
        {
            InstallLogger.Instance.Write("Starting install process");
            PrinterInstaller.RemoveUninstallKey();
            InstallLogger.Instance.Write("Creating folder in Program Files");
            PrinterInstaller.CreateProgramFilesDir();
            InstallLogger.Instance.Write("Installing RedMon");
            if (!PrinterInstaller.InstallRedmon())
            {
                this.showErrorMessageBox("Could not install the Print Monitor.");
                return;
            }
            this.updateProgress();
            InstallLogger.Instance.Write("Importing registry keys for printer port");
            if (!PrinterInstaller.ConfigurePort())
            {
                this.showErrorMessageBox("Could not configure the print port.");
                return;
            }
            this.updateProgress();
            InstallLogger.Instance.Write("Installing virtual printer");
            if (!PrinterInstaller.InstallPrinter())
            {
                this.showErrorMessageBox("Could not install the printer. Most likely the appropriate driver could not be found.");
                return;
            }
            this.updateProgress();
            InstallLogger.Instance.Write("Extracting JRE and support files");
            if (!PrinterInstaller.InstallPLP())
            {
                this.showErrorMessageBox("Could not install the Java Runtime Environment or the Pitt Lab Printer jar file.");
                return;
            }
            this.updateProgress();
            InstallLogger.Instance.Write("Making shortcuts in Start Menu");
            PrinterInstaller.MakeShortcuts();
            InstallLogger.Instance.Write("Writing uninstall keys");
            PrinterInstaller.SetUninstallKeys();
            InstallLogger.Instance.Write("Done!");
            InstallLogger.Instance.Close();
            this.showInfoMessageBox("Done!", "Installation successful.\r\nView the help file in Start Menu -> Programs - > Pitt Lab Printer -> Help\r\nfor information on how to print.");
            Application.Exit();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.button1.Enabled = false;
            ThreadPool.QueueUserWorkItem(new WaitCallback(Install));
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.button2.Enabled = false;
            ThreadPool.QueueUserWorkItem(new WaitCallback(Uninstall));
        }

        private void Uninstall(object o)
        {
            PrinterRemover.RemovePrinter();
            this.updateProgress();
            PrinterRemover.RemovePort();
            this.updateProgress();
            PrinterRemover.RemoveRedmon();
            this.updateProgress();
            PrinterRemover.RemoveShortcuts();
            this.updateProgress();
            PrinterRemover.RemoveFolder();
            PrinterRemover.RemoveUninstallKeys();
            this.updateProgress();
            this.showInfoMessageBox("Done!", "Uninstallation Successful. All remaining files will be removed after next reboot.");
            Application.Exit();
        }

    }
}