using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using System.IO;
using Microsoft.Win32;

namespace Pitt_Lab_Printer_Setup
{
    class PrinterInstaller
    {
        private const string WIN2K_PRINTER_NAME = "Apple Color LW 12/660 PS";
        private const string WINXPVISTA_PRINTER_NAME = "Canon PS-NX90";
        private const string WIN7_PRINTER_NAME = "Xerox Phaser 6180N PS";

        private const string SOFTWARE_VERSION = "1.7";

        private static readonly string[] ZERO_VALUES = { "Output", "StartShell", "ServerRunning", "DebugLevel", "LogFileUse", "LogFileDebug", "PrintError", "CleanUpDocNames", "UseRunDll32", "DefaultFileName", "UseRPC", "RPCDefaultProtocol", "RPCUseDefaultPort", "RPCDefaultPort", "RPCSendError", "RPCFileTransfer", "PDFLevel", "Encrypt", "DisableAll", "PromptSecOptions", "DisablePrint", "DisableModify", "DisableCopy", "DisableAnnotations", "DisableExtract", "DisableAssemble", "DisableHQPrint", "FileNumber" };
        private static readonly string[] EMPTY_VALUES = { "AfterworksCommand", "AfterworksArguments", "LogFileName", "ExternalSaveAs", "FilenameTemplate", "DefaultExtension", "ServerProgram", "ServerPgmParam", "OwnerPassword", "UserPassword" };

        public static void RemoveUninstallKey()
        {
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce", true);
            if (key.GetValue("PLPUninstall") != null)
            {
                key.DeleteValue("PLPUninstall");
            }
        }

        public static void MakeShortcuts()
        {
            String s = Path.Combine(Paths.StartMenuProgramsPath, "Pitt Lab Printer");
            if (!Directory.Exists(s))
            {
                Directory.CreateDirectory(s);
            }
            IWshRuntimeLibrary.IWshShell_Class shell = new IWshRuntimeLibrary.IWshShell_Class();
            IWshRuntimeLibrary.IWshShortcut shortcut = (IWshRuntimeLibrary.IWshShortcut)shell.CreateShortcut(Path.Combine(s, "Pitt Lab Printer.lnk"));
            shortcut.TargetPath = Path.Combine(Paths.PLPProgramFiles, @"jre\bin\java.exe");
            shortcut.Arguments = "-jar \"" + Path.Combine(Paths.PLPProgramFiles, @"PittLabPrinter.jar") + "\"";
            shortcut.Save();
            shortcut = (IWshRuntimeLibrary.IWshShortcut)shell.CreateShortcut(Path.Combine(s, "Help.lnk"));
            shortcut.TargetPath = Path.Combine(Paths.PLPProgramFiles, "help.html");
            shortcut.Save();
            shortcut = (IWshRuntimeLibrary.IWshShortcut)shell.CreateShortcut(Path.Combine(s, "License.lnk"));
            shortcut.TargetPath = Path.Combine(Paths.PLPProgramFiles, "license.html");
            shortcut.Save();
        }

        private static void extractRedmon()
        {
            Extractor.Extract("redmon" + Utilities.GetArchitecture() + ".zip", Paths.PLPProgramFiles);
        }

        public static bool InstallRedmon()
        {
            try
            {
                extractRedmon();
                Process p = new Process();
                p.StartInfo.FileName = Path.Combine(Paths.PLPProgramFiles, @"redmon\setup.exe");
                p.StartInfo.Arguments = "/quiet";
                p.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
                p.Start();
                p.WaitForExit();
            }
            catch (Exception e)
            {
                InstallLogger.Instance.Write(e.ToString());
                return false;
            }
            return true;
        }

        public static bool CreateProgramFilesDir()
        {
            try
            {
                bool isRunningFromProgFiles = (Path.GetDirectoryName(Application.ExecutablePath) == Paths.PLPProgramFiles);
                if (!isRunningFromProgFiles && Directory.Exists(Paths.PLPProgramFiles))
                {
                    Directory.Delete(Paths.PLPProgramFiles, true);
                }
                Directory.CreateDirectory(Paths.PLPProgramFiles);
            }
            catch (Exception e)
            {
                InstallLogger.Instance.Write(e.ToString());
                return false;
            }
            return true;
        }

        public static bool ConfigurePort()
        {
            return SetRegPortValues();
        }

        private static bool SetRegPortValues()
        {
            try
            {
                RegistryKey reg = Registry.LocalMachine.OpenSubKey(@"SYSTEM\CurrentControlSet\Control\Print\Monitors\Redirected Port", true);
                if (reg == null)
                {
                    return false;
                }
                if (reg.OpenSubKey("Ports") == null)
                {
                    reg.CreateSubKey("Ports");
                }
                reg = reg.OpenSubKey("Ports", true);
                if (reg.OpenSubKey("PLP:", false) != null)
                {
                    reg.DeleteSubKeyTree("PLP:");
                }
                reg.CreateSubKey("PLP:");
                reg = reg.OpenSubKey("PLP:", true);
                foreach (string val in ZERO_VALUES)
                {
                    reg.SetValue(val, 0);
                }
                foreach (string val in EMPTY_VALUES)
                {
                    reg.SetValue(val, String.Empty);
                }
                reg.SetValue("Printer", "Pitt Lab Printer");
                reg.SetValue("Description", "Pitt Lab Printer Port");
                reg.SetValue("Command", Path.Combine(Paths.PLPProgramFiles, @"jre\bin\java.exe"));
                reg.SetValue("Arguments", "-jar \"" + Path.Combine(Paths.PLPProgramFiles, "PittLabPrinter.jar") + "\" --file --stdin");
                reg.SetValue("PriorityPrim", 1);
                reg.SetValue("PrioritySec", 1);
                reg.SetValue("Delay", 1800);
                reg.SetValue("RunUser", 1);
                reg.SetValue("ShowWindow", 2);
            }
            catch (Exception e)
            {
                InstallLogger.Instance.Write(e.ToString());
                return false;
            }
            return true;
        }

        private static string getPrinterName()
        {
            Version osVersion = Environment.OSVersion.Version;
            if (osVersion.Major == 5 && osVersion.Minor == 0) // Win 2K
            {
                return PrinterInstaller.WIN2K_PRINTER_NAME;
            }
            if (osVersion.Major < 6) // XP or W2K3Server
            {
                return PrinterInstaller.WINXPVISTA_PRINTER_NAME;
            }
            if (osVersion.Major == 6 && osVersion.Minor == 0) // Vista or W2K8Server
            {
                return PrinterInstaller.WINXPVISTA_PRINTER_NAME;
            }
            return PrinterInstaller.WIN7_PRINTER_NAME; // anything newer (such as Win7)
        }

        public static bool InstallPrinter()
        {
            try
            {
                Process p = new Process();
                ProcessStartInfo ps = p.StartInfo;
                ps.FileName = "rundll32.exe";
                String infPath = Path.Combine(Environment.GetEnvironmentVariable("SYSTEMROOT"), @"inf\ntprint.inf");
                string printerDriverName = PrinterInstaller.getPrinterName();

                ps.Arguments = String.Format("printui.dll,PrintUIEntry /if /b \"Pitt Lab Printer\" /f \"{0}\" /r \"plp:\" /m \"{1}\"", infPath, printerDriverName);
                ps.WindowStyle = ProcessWindowStyle.Normal;
                p.Start();
                p.WaitForExit();
                InstallLogger.Instance.Write("Exit code for printui.dll was " + p.ExitCode);
                if (p.ExitCode != 0)
                {
                    return false;
                }
            }
            catch (Exception e)
            {
                InstallLogger.Instance.Write(e.ToString());
                return false;
            } 
            return true;
        }

        public static bool InstallPLP()
        {  
            try
            {
                Extractor.Extract("jre.zip", Paths.PLPProgramFiles);
                Extractor.WriteResourceToProgFiles("help.html");
                Extractor.WriteResourceToProgFiles("PittLabPrinter.jar");
                Extractor.WriteResourceToProgFiles("license.html");
                if (Path.GetDirectoryName(Application.ExecutablePath) != Paths.PLPProgramFiles)
                {
                    File.Copy(System.Windows.Forms.Application.ExecutablePath, Path.Combine(Paths.PLPProgramFiles, "Pitt Lab Printer Setup.exe"));
                }
            }
            catch (Exception e)
            {
                InstallLogger.Instance.Write(e.ToString());
                return false;
            }
            return true;
        }

        public static void SetUninstallKeys()
        {
            try
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall", true);
                RegistryKey subKey = key.OpenSubKey("Pitt Lab Printer", true);
                if (subKey == null)
                {
                    subKey = key.CreateSubKey("Pitt Lab Printer");
                }
                subKey.SetValue("DisplayName", "Pitt Lab Printer");
                subKey.SetValue("DisplayIcon", Path.Combine(Paths.PLPProgramFiles, "Pitt Lab Printer Setup.exe"));
                subKey.SetValue("DisplayVersion", SOFTWARE_VERSION);
                subKey.SetValue("InstallLocation", Paths.PLPProgramFiles);
                subKey.SetValue("Publisher", "The SCK");
                subKey.SetValue("UninstallString", "\"" + Path.Combine(Paths.PLPProgramFiles, "Pitt Lab Printer Setup.exe") + "\"");
                subKey.Close();
                key.Close();
            }
            catch (Exception e)
            {
                InstallLogger.Instance.Write(e.ToString());
            }
        }

    }
}
